local SorhaQuestLog = LibStub("AceAddon-3.0"):GetAddon("SorhaQuestLog")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local MODNAME = "ActivitiesTracker"
local ActivitiesTracker = SorhaQuestLog:NewModule(MODNAME, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "LibSink-2.0")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

local fraMinionAnchor = nil
local blnMinionInitialized = false
local blnMinionUpdating = false

local strButtonPrefix = MODNAME .. "Button"


local intNumberUsedButtons = 0
local tblButtonCache = {}
local tblUsingButtons = {}

local strMinionTitleColour = "|cffffffff"
local strActivityTitleColour = "|cffffffff"
local strActivityDescriptionColour = "|cffffffff"
local strActivityRequirementColour = "|cffffffff"
local strActivityRequirementDoneColour = "|cff33ff33"
local strActivityReqirementUndoneColour = "|cffDD2222"


local activitiesData = nil;

local dicOutlines = {
	[""] = NONE,
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
	["MONOCHROMEOUTLINE"] = L["Monochrome Outline"],
}

--Defaults
local db
local dbCore
local dbChar;
local defaults = {
	profile = {
		MinionParent = "UIParent",
		MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
		MinionScale = 1,
		MinionLocked = false,
		MinionWidth = 220,
		MinionCollapseToLeft = false,
		MoveTooltipsRight = false,
		ShowTitle = true,
		AutoHideTitle = false,
		MaxRequirementsEachActivity = 0,
		GrowUpwards = false,
		ActivityContentIndent = 5,
		PaddingAfterActivity = 3,
		Fonts = {
			MinionTitleFontSize = 11,
			MinionTitleFont = "framd",
			MinionTitleFontOutline = "",
			MinionTitleFontShadowed = true,
			
			ActivityTitleFontSize = 11,
			ActivityTitleFont = "framd",
			ActivityTitleFontOutline = "",
			ActivityTitleFontShadowed = true,
						
			ActivityRequirementFontSize = 11,
			ActivityRequirementFont = "framd",
			ActivityRequirementFontOutline = "",
			ActivityRequirementFontShadowed = true,			
		},
		Colours = {
			MinionTitleColour = {r = 0, g = 1, b = 0, a = 1},
			ActivityTitleColour = {r = 0, g = 1, b = 0, a = 1},
			ActivityDescriptionColour = {r = 0.8, g = 1, b = 0.6, a = 1},			
			ActivityRequirementColour = {r = 1, g = 1, b = 1, a = 1},			
			ActivityRequirementUndoneColour = {r = 0.8, g = 0.1, b = 0.1, a = 1},			
			MinionBackGroundColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			MinionBorderColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
		}
	},
	char = {
		ActivitiesCollapsedState = {},
	}
}

-- Options
local options
local function getOptions()
	if not options then
		options = {
			name = L["Activities Settings"],
			type = "group",
			childGroups = "tab",
			order = 2,
			arg = MODNAME,
			args = {
				Main = {
					name = "Main",
					type = "group",
					order = 1,
					args = {
						enabled = {
							order = 1,
							type = "toggle",
							name = L["Enable Minion"],
							get = function() return SorhaQuestLog:GetModuleEnabled(MODNAME) end,
							set = function(info, value) 
								SorhaQuestLog:SetModuleEnabled(MODNAME, value) 
								ActivitiesTracker:MinionAnchorUpdate(false)
							end,
						},
						MinionLockedToggle = {
							name = L["Lock Minion"],
							type = "toggle",
							get = function() return db.MinionLocked end,
							set = function()
								db.MinionLocked = not db.MinionLocked
								ActivitiesTracker:MinionAnchorUpdate(false)
							end,
							order = 2,
						},
						ShowTitleToggle = {
							name = L["Show Minion Title"],
							type = "toggle",
							get = function() return db.ShowTitle end,
							set = function()
								db.ShowTitle = not db.ShowTitle
								ActivitiesTracker:UpdateMinion()
							end,
							order = 3,
						},
						AutoHideTitleToggle = {
							name = L["Auto Hide Minion Title"],
							desc = L["Hide the title when there is nothing to display"],
							type = "toggle",
							disabled = function() return not(db.ShowTitle) end,
							get = function() return db.AutoHideTitle end,
							set = function()
								db.AutoHideTitle = not db.AutoHideTitle
								ActivitiesTracker:UpdateMinion(false)
							end,
							order = 4,
						},
						GrowUpwardsToggle = {
							name = L["Grow Upwards"],
							desc = L["Minions grows upwards from the anchor"],
							type = "toggle",
							get = function() return db.GrowUpwards end,
							set = function()
								db.GrowUpwards = not db.GrowUpwards
								ActivitiesTracker:UpdateMinion()
							end,
							order = 5,
						},						
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current activities is less then the max width\nNote: Doesn't work well with activities that word wrap"],
							type = "toggle",
							get = function() return db.MinionCollapseToLeft end,
							set = function()
								db.MinionCollapseToLeft = not db.MinionCollapseToLeft
								ActivitiesTracker:UpdateMinion()
							end,
							order = 86,
						},
						MoveTooltipsRightToggle = {
							name = L["Tooltips on right"],
							desc = L["Moves the tooltips to the right"],
							type = "toggle",
							get = function() return db.MoveTooltipsRight end,
							set = function()
								db.MoveTooltipsRight = not db.MoveTooltipsRight
								ActivitiesTracker:UpdateMinion()
							end,
							order = 7,
						},
						MinionScaler = {
							order = 10,
							name = L["Minion Scale"],
							desc = L["Adjust the scale of the minion"],
							type = "range",
							min = 0.5, max = 2, step = 0.05,
							isPercent = false,
							get = function() return db.MinionScale end,
							set = function(info, value)
								db.MinionScale = value
								ActivitiesTracker:MinionAnchorUpdate(true)
							end,
						},
						MinionWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Adjust the width of the minion"],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.MinionWidth end,
							set = function(info, value)
								db.MinionWidth = value
								ActivitiesTracker:UpdateMinion()
							end,
						},		
						MinionParent = {
							name = L["Minion Anchor Point"],
							desc = L["The minion to anchor this minion to"],
							type = "select",
							order = 12,
							values = function() return SorhaQuestLog:GetPossibleParents(fraMinionAnchor)end,
							get = function() return db.MinionParent  end,
							set = function(info, value)
								db.MinionParent = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivitiesMinionReset = {
							order = 14,
							type = "execute",
							name = L["Reset Minion Position"],
							desc = L["Resets Activities Minions position"],
							func = function()
								db.MinionLocation.Point = "CENTER"
								db.MinionLocation.RelativePoint =  "CENTER"
								db.MinionLocation.X = 0
								db.MinionLocation.Y = 0
								ActivitiesTracker:MinionAnchorUpdate(true)
							end,
						},
						ActivitiesSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 30,
						},
						ActivitiesSpacerHeader = {
							name = L["Activities Settings"],
							type = "header",
							order = 31,
						},
						MaxNumRequirements = {
							order = 34,
							name = L["Requirements # Cap (0 = All)"],
							desc = L["# Requirements shown per Activity. Set to 0 to display all Requirements"],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.MaxRequirementsEachActivity end,
							set = function(info, value)
								db.MaxRequirementsEachActivity = value
								if (blnMinionUpdating == false) then
									blnMinionUpdating = true
									ActivitiesTracker:ScheduleTimer("UpdateMinion", 0.25)
								end
							end,
						},
						ContentIndent = {
							order = 35,
							name = L["Content Indent"],
							desc = L["Controls the level of indentation for the activities content"],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.ActivityContentIndent end,
							set = function(info, value)
								db.ActivityContentIndent = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityAfterPadding = {
							order = 36,
							name = L["Activity Padding"],
							desc = L["The amount of extra padding after an Activity before the next."],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.PaddingAfterActivity end,
							set = function(info, value)
								db.PaddingAfterActivity = value
								ActivitiesTracker:UpdateMinion()
							end,
						}
					}		
				},
				Fonts = {
					name = "Fonts",
					type = "group",
					order = 5,
					args = {
						HeaderTitleFont = {
							name = L["Minion Title Font Settings"],
							type = "header",
							order = 41,
						},
						MinionTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 42,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.MinionTitleFont end,
							set = function(info, value)
								db.Fonts.MinionTitleFont = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						MinionTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 43,
							values = dicOutlines,
							get = function() return db.Fonts.MinionTitleFontOutline end,
							set = function(info, value)
								db.Fonts.MinionTitleFontOutline = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						MinionTitleFontSizeSelect = {
							order = 44,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.MinionTitleFontSize end,
							set = function(info, value)
								db.Fonts.MinionTitleFontSize = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						MinionTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.MinionTitleFontShadowed end,
							set = function()
								db.Fonts.MinionTitleFontShadowed = not db.Fonts.MinionTitleFontShadowed
								ActivitiesTracker:UpdateMinion()
							end,
							order = 45,
						},
						
						HeaderFontsSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderFonts = {
							name = L["Header Font Settings"],
							type = "header",
							order = 51,
						},
						ActivityTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 52,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.ActivityTitleFont end,
							set = function(info, value)
								db.Fonts.ActivityTitleFont = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 53,
							values = dicOutlines,
							get = function() return db.Fonts.ActivityTitleFontOutline end,
							set = function(info, value)
								db.Fonts.ActivityTitleFontOutline = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityTitleFontSizeSize = {
							order = 54,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.ActivityTitleFontSize end,
							set = function(info, value)
								db.Fonts.ActivityTitleFontSize = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.ActivityTitleFontShadowed end,
							set = function()
								db.Fonts.ActivityTitleFontShadowed = not db.Fonts.ActivityTitleFontShadowed
								ActivitiesTracker:UpdateMinion()
							end,
							order = 55,
						},
						
						RequirementFontsSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 60,
						},
						RequirementFonts = {
							name = L["Requirement Font Settings"],
							type = "header",
							order = 61,
						},
						ActivityRequirementFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 62,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.ActivityRequirementFont end,
							set = function(info, value)
								db.Fonts.ActivityRequirementFont = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityRequirementFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 63,
							values = dicOutlines,
							get = function() return db.Fonts.ActivityRequirementFontOutline end,
							set = function(info, value)
								db.Fonts.ActivityRequirementFontOutline = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityRequirementFontSize = {
							order = 64,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.ActivityRequirementFontSize end,
							set = function(info, value)
								db.Fonts.ActivityRequirementFontSize = value
								ActivitiesTracker:UpdateMinion()
							end,
						},
						ActivityRequirementFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.ActivityRequirementFontShadowed end,
							set = function()
								db.Fonts.ActivityRequirementFontShadowed = not db.Fonts.ActivityRequirementFontShadowed
								ActivitiesTracker:UpdateMinion()
							end,
							order = 65,
						},
					}
				},
				Colours = {
					name = "Colours",
					type = "group",
					order = 6,
					args = {
						HeaderColourSettings = {
							name = L["Colour Settings"],
							type = "header",
							order = 80,
						},
						MinionTitleColour = {
							name = L["Minion Title"],
							desc = L["Sets the color for Minion Title"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Colours.MinionTitleColour.r, db.Colours.MinionTitleColour.g, db.Colours.MinionTitleColour.b, db.Colours.MinionTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionTitleColour.r = r
									db.Colours.MinionTitleColour.g = g
									db.Colours.MinionTitleColour.b = b
									db.Colours.MinionTitleColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 81,
						},
						ActivityTitleColour = {
							name = L["Activity Titles"],
							desc = L["Sets the color for Activity Titles"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ActivityTitleColour.r, db.Colours.ActivityTitleColour.g, db.Colours.ActivityTitleColour.b, db.Colours.ActivityTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ActivityTitleColour.r = r
									db.Colours.ActivityTitleColour.g = g
									db.Colours.ActivityTitleColour.b = b
									db.Colours.ActivityTitleColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 82,
						},
						ActivityDescriptionColour = {
							name = L["Activity Description"],
							desc = L["Sets the color for Activity Description"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ActivityDescriptionColour.r, db.Colours.ActivityDescriptionColour.g, db.Colours.ActivityDescriptionColour.b, db.Colours.ActivityDescriptionColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ActivityDescriptionColour.r = r
									db.Colours.ActivityDescriptionColour.g = g
									db.Colours.ActivityDescriptionColour.b = b
									db.Colours.ActivityDescriptionColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 83,
						},
						ActivityRequirementColour = {
							name = L["Activity Requirements"],
							desc = L["Sets the color for Activity Requirements"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ActivityRequirementColour.r, db.Colours.ActivityRequirementColour.g, db.Colours.ActivityRequirementColour.b, db.Colours.ActivityRequirementColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ActivityRequirementColour.r = r
									db.Colours.ActivityRequirementColour.g = g
									db.Colours.ActivityRequirementColour.b = b
									db.Colours.ActivityRequirementColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 84,
						},
						ActivityRequirementUndoneColourSelector = {
							name = L["Requirement In Progress"],
							desc = L["Sets the color for In Progress Activity Requirements"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ActivityRequirementUndoneColour.r, db.Colours.ActivityRequirementUndoneColour.g, db.Colours.ActivityRequirementUndoneColour.b, db.Colours.ActivityRequirementUndoneColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ActivityRequirementUndoneColour.r = r
									db.Colours.ActivityRequirementUndoneColour.g = g
									db.Colours.ActivityRequirementUndoneColour.b = b
									db.Colours.ActivityRequirementUndoneColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 85,
						},
						MinionBackGroundColour = {
							name = L["Background Colour"],
							desc = L["Sets the color of the minions background"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionBackGroundColour.r = r
									db.Colours.MinionBackGroundColour.g = g
									db.Colours.MinionBackGroundColour.b = b
									db.Colours.MinionBackGroundColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 89,
						},
						MinionBorderColour = {
							name = L["Border Colour"],
							desc = L["Sets the color of the minions border"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionBorderColour.r = r
									db.Colours.MinionBorderColour.g = g
									db.Colours.MinionBorderColour.b = b
									db.Colours.MinionBorderColour.a = a
									ActivitiesTracker:HandleColourChanges()
								end,
							order = 90,
						},
					}
				},
			}
		}
	end

	return options
end


local function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

local SQLActivityRequirement = {};
SQLActivityRequirement.__index = SQLActivityRequirement; 

function SQLActivityRequirement:new(requirementKey, requirement, activityID) 
	local self = {};
	setmetatable(self, SQLActivityRequirement);

	self.Index = requirementKey;
	self.ActivitiesID = activityID
	self.Text = ""
	self.Requirement = requirement;
	self.IsComplete = requirement.completed;
	self.Changed = false;

	self:Update(self.Requirement);

	return self;
end

function SQLActivityRequirement:Update(requirement)
	self.Requirement = requirement;
	self.IsComplete = requirement.completed;
	
	self.Changed = false;
	if (self.Text ~= requirement.requirementText) then
		self.Changed = true;
	end
	self.Text = requirement.requirementText;
end

function SQLActivityRequirement:Render(colourCompletion)
	local strOutput = strActivityRequirementColour;


	if (colourCompletion == true and self.IsComplete == true) then
		strOutput = strActivityRequirementDoneColour;
	end
	strOutput = strOutput ..  self.Text .. "|r";
	
	return strOutput;
end



local SQLActivity = {};
SQLActivity.__index = SQLActivity; 

function SQLActivity:new(activityID) 
	local self = {};
	setmetatable(self, SQLActivity);

	self.ID = activityID;
	self.Name = "";
	self.Description = "";

	self.RequirementsList = {};
	self.RequirementsCount = 0;

	self.Changed = false;
	self.Keep = false;

	self.IsCollapsed = false;
	if (dbChar.ActivitiesCollapsedState[self.ID] == true) then
		self.IsCollapsed = true;
	end

	self:Update();

	return self;
end

function SQLActivity:Update()
	self.Changed = false;
	self.Keep = true;


	local activityInfo = C_PerksActivities.GetPerksActivityInfo(self.ID);
	self.Name = activityInfo.activityName;
	self.Description = activityInfo.description;
	self.Keep = not activityInfo.completed;
	
	for requirementKey, requirement in pairs(activityInfo.requirementsList) do
		if (self.RequirementsList[requirementKey] == nil) then
			self.RequirementsList[requirementKey] = SQLActivityRequirement:new(requirementKey, requirement, self.ID);
			self.RequirementsCount = self.RequirementsCount + 1;
		else
			self.RequirementsList[requirementKey]:Update(requirement);
		end
	end

	--If any requirement changed, then changed

	return self;
end

function SQLActivity:Render()
	return strActivityTitleColour .. self.Name .. "|r";
end

function SQLActivity:RenderDescription()
	if(self.Description ~= "") then
		return strActivityDescriptionColour .. self.Description  .. "|r";
	end
	return "";
end

local SQLActivitiesData = {};
SQLActivitiesData.__index = SQLActivitiesData; 


function SQLActivitiesData:new() 
	local self = {};
	setmetatable(self, SQLActivitiesData);

	self.FirstUpdate = true;
	self.EntryCount = 0;

	self.ActivitiesCount = 0;
	self.ActivitiesList = {};

	self.Changed = false;
	self:Update();
	return self;
end

function SQLActivitiesData:Update()
	--self.Changed = false;

	for activitieKey, activity in pairs(self.ActivitiesList) do
    	activity.Keep = false;
	end

	local trackedActivities = C_PerksActivities.GetTrackedPerksActivities().trackedIDs;

	for i = 1, #trackedActivities do
		local activityID = trackedActivities[i];
		if (self.ActivitiesList[activityID] == nil) then
			self.ActivitiesList[activityID] = SQLActivity:new(activityID);
			self.ActivitiesCount = self.ActivitiesCount + 1;
		else
			self.ActivitiesList[activityID]:Update();
		end
	end

	for activitieKey, activity in pairs(self.ActivitiesList) do
    	if (activity.Keep == false) then
    		self.ActivitiesList[activity.ID] = nil;
    		self.ActivitiesCount = self.ActivitiesCount - 1;
			MonthlyActivitiesObjectiveTracker_UntrackPerksActivity(_, activity.ID)
    	end
	end
	self.FirstUpdate = false;
end

--Inits
function ActivitiesTracker:OnInitialize()
	self.db = SorhaQuestLog.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	dbChar = self.db.char
	dbCore = SorhaQuestLog.db.profile
	self:SetEnabledState(SorhaQuestLog:GetModuleEnabled(MODNAME))
	SorhaQuestLog:RegisterModuleOptions(MODNAME, getOptions, L["Activities Tracker"])
	
	self:UpdateColourStrings()
	self:MinionAnchorUpdate(true)
end

function ActivitiesTracker:OnEnable()
	self:RegisterEvent("PERKS_ACTIVITIES_TRACKED_UPDATED");
	self:RegisterEvent("PERKS_ACTIVITY_COMPLETED");
	
	self:MinionAnchorUpdate(false)
	self:UpdateMinion()
end

function ActivitiesTracker:OnDisable()
	self:UnregisterEvent("PERKS_ACTIVITIES_TRACKED_UPDATED")	
	self:UnregisterEvent("PERKS_ACTIVITY_COMPLETED")	
	self:MinionAnchorUpdate(true)
	self:UpdateMinion()
end

function ActivitiesTracker:Refresh()
	db = self.db.profile
	dbCore = SorhaQuestLog.db.profile
	
	self:HandleColourChanges()
	self:MinionAnchorUpdate(true)
end

--Events/handlers
function ActivitiesTracker:PERKS_ACTIVITIES_TRACKED_UPDATED(...)	
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then 
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

function ActivitiesTracker:PERKS_ACTIVITY_COMPLETED(...)
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then 
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

--Buttons
function ActivitiesTracker:GetMinionButton()
	local objButton = SorhaQuestLog:GetLogButton()
	objButton:SetParent(fraMinionAnchor)
	tinsert(tblUsingButtons, objButton);
	
	objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);

	return objButton
end


function ActivitiesTracker:GetMinionActivityButton(activityInstance)
	local objButton = ActivitiesTracker:GetMinionButton(activityInstance)
	objButton.ActivityInstance = activityInstance;

	-- Set buttons title text
	objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.ActivityTitleFont), db.Fonts.ActivityTitleFontSize, db.Fonts.ActivityTitleFontOutline)
	if (db.Fonts.ActivityTitleFontShadowed == true) then
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	objButton.objFontString1:SetText(objButton.ActivityInstance:Render())
	if (objButton.objFontString1:GetWidth() > db.MinionWidth) then
		objButton.objFontString1:SetWidth(db.MinionWidth)
	end

	
	local currentHeight = 0;
	local currentWidth = 0;

	currentWidth = objButton.objFontString1:GetWidth();

	if (db.Fonts.ActivityTitleFontOutline == "THICKOUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 2;
	elseif (db.Fonts.ActivityTitleFontOutline == "OUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 1;
	else
		currentHeight = objButton.objFontString1:GetHeight();
	end
	

	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		if (button == "LeftButton") then
			if (IsControlKeyDown()) then
				self.ActivityInstance.IsCollapsed = not self.ActivityInstance.IsCollapsed;
				dbChar.ActivitiesCollapsedState[self.ActivityInstance.ID] = self.ActivityInstance.IsCollapsed;
				ActivitiesTracker:UpdateMinion()
				
			elseif (IsShiftKeyDown()) then
				ChatEdit_InsertLink(C_PerksActivities.GetPerksActivityChatLink(self.ActivityInstance.ID));
			else
				if (not EncounterJournal) then
					EncounterJournal_LoadUI();
				end

				MonthlyActivitiesFrame_OpenFrameToActivity(self.ActivityInstance.ID)
			end
		else
			MonthlyActivitiesObjectiveTracker_UntrackPerksActivity(_,self.ActivityInstance.ID)
		end
	end)

	objButton:SetScript("OnEnter", function(self)
		if (db.MoveTooltipsRight == true) then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
		else 
			GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
		end

		GameTooltip:SetText(self.ActivityInstance:Render(), 0, 1, 0, 1);
		
		GameTooltip:AddLine(self.ActivityInstance:RenderDescription(), 0.5, 0.5, 0.5, 1);
		if (self.ActivityInstance.RequirementsCount > 0) then
			GameTooltip:AddLine(" ", 0.5, 0.5, 0.5, 1);
		end
		for i, criteria in spairs(self.ActivityInstance.RequirementsList, function(t,a,b) return t[b].Text > t[a].Text end) do
			GameTooltip:AddLine(criteria:Render(true), 0.5, 0.5, 0.5, 1);
		end
			
		GameTooltip:Show();
	end)

	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)

	objButton.Width = currentWidth;
	objButton.Height = currentHeight;
	objButton:SetWidth(db.MinionWidth);	
	return objButton
end

function ActivitiesTracker:GetMinionRequirementButton(requirementInstance)
	local objButton = ActivitiesTracker:GetMinionButton(activitieInstance)
	objButton.RequirementInstance = requirementInstance;

	objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.ActivityRequirementFont), db.Fonts.ActivityRequirementFontSize, db.Fonts.ActivityRequirementFontOutline)
	if (db.Fonts.ActivityRequirementFontShadowed == true) then
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	local currentHeight = 0;
	local currentWidth = 0;
	objButton.objFontString1:SetText(objButton.RequirementInstance:Render());
	

	if (objButton.objFontString1:GetWidth() > (db.MinionWidth - db.ActivityContentIndent)) then
		objButton.objFontString1:SetWidth(db.MinionWidth - db.ActivityContentIndent)
	end

	currentWidth = objButton.objFontString1:GetWidth();

	if (db.Fonts.ActivityRequirementFontOutline == "THICKOUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 1.5;
	elseif (db.Fonts.ActivityRequirementFontOutline == "OUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 0.5;
	else
		currentHeight = objButton.objFontString1:GetHeight();
	end	



	objButton.Width = currentWidth;
	objButton.Height = currentHeight;

	objButton:SetHeight(objButton.Height);
	objButton:SetWidth(db.MinionWidth);
	return objButton
end

function ActivitiesTracker:GetMinionRequirementCappedButton()
	local objButton = ActivitiesTracker:GetMinionButton()

	
	objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.ActivityRequirementFont), db.Fonts.ActivityRequirementFontSize, db.Fonts.ActivityRequirementFontOutline)
	if (db.Fonts.ActivityRequirementFontShadowed == true) then
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	local currentHeight = 0;
	local currentWidth = 0;

	objButton.objFontString1:SetText("...");
	if (objButton.objFontString1:GetWidth() > (db.MinionWidth - db.ActivityContentIndent)) then
		objButton.objFontString1:SetWidth(db.MinionWidth - db.ActivityContentIndent)
	end

	currentWidth = objButton.objFontString1:GetWidth();

	if (db.Fonts.ActivityRequirementFontOutline == "THICKOUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 1.5;
	elseif (db.Fonts.ActivityRequirementFontOutline == "OUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 0.5;
	else
		currentHeight = objButton.objFontString1:GetHeight();
	end	
	
	objButton.Width = currentWidth;
	objButton.Height = currentHeight;

	objButton:SetHeight(objButton.Height);
	objButton:SetWidth(db.MinionWidth);
	return objButton
end

function ActivitiesTracker:RecycleMinionButton(objButton)
	objButton.Height = 0;
	objButton.ActivityInstance = nil;
	objButton.RequirementInstance = nil;

	SorhaQuestLog:RecycleLogButton(objButton);
end


--Minion
function ActivitiesTracker:CreateMinionLayout()
	fraMinionAnchor = SorhaQuestLog:doCreateFrame("FRAME","SQLActivitiesMinionAnchor",UIParent,100,20,1,"BACKGROUND",1, db.MinionLocation.Point, UIParent, db.MinionLocation.RelativePoint, db.MinionLocation.X, db.MinionLocation.Y, 1, "BackdropTemplate")
	fraMinionAnchor:SetMovable(true)
	fraMinionAnchor:SetClampedToScreen(true)
	fraMinionAnchor:RegisterForDrag("LeftButton")
	fraMinionAnchor:SetScript("OnDragStart", fraMinionAnchor.StartMoving)
	fraMinionAnchor:SetScript("OnDragStop",  function(self)
		fraMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraMinionAnchor:GetPoint()
		db.MinionLocation.Point = strPoint
		db.MinionLocation.RelativePoint = strRelativePoint
		db.MinionLocation.X = intPosX
		db.MinionLocation.Y = intPosY
	end)
	fraMinionAnchor:SetScript("OnEnter", function(self) 
		if (dbCore.Main.ShowHelpTooltips == true) then
			if (db.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Activities Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Activities minion when it is unlocked."], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraMinionAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	
	fraMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraMinionAnchor:SetBackdropColor(0, 0, 1, 0)
	fraMinionAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	
	-- Activitiess Anchor
	fraMinionAnchor.fraActivitiesAnchor = SorhaQuestLog:doCreateLooseFrame("FRAME","SQLQuestsAnchor",fraMinionAnchor, fraMinionAnchor:GetWidth(),1,1,"LOW",1,1,"BackdropTemplate")
	fraMinionAnchor.fraActivitiesAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0);
	fraMinionAnchor.fraActivitiesAnchor:SetBackdropColor(0, 0, 0, 0)
	fraMinionAnchor.fraActivitiesAnchor:SetBackdropBorderColor(0,0,0,0)
	fraMinionAnchor.fraActivitiesAnchor:SetAlpha(0)
	
	
	-- Title Fontstring
	fraMinionAnchor.objFontString = fraMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraMinionAnchor.objFontString:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT",0, 0);
	fraMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.MinionTitleFont), db.Fonts.MinionTitleFontSize, db.Fonts.MinionTitleFontOutline)
	if (db.Fonts.MinionTitleFontShadowed == true) then
		fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	fraMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraMinionAnchor.objFontString:SetJustifyV("TOP")
	fraMinionAnchor.objFontString:SetText("");
	fraMinionAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraMinionAnchor.BorderFrame = SorhaQuestLog:doCreateFrame("FRAME","SQLActivitiesMinionBorder", fraMinionAnchor, 100,20,1,"BACKGROUND",1, "TOPLEFT", fraMinionAnchor, "TOPLEFT", -8, 6, 1, "BackdropTemplate")
	fraMinionAnchor.BorderFrame:SetBackdrop({bgFile = LSM:Fetch("background", dbCore.BackgroundTexture), tile = false, tileSize = 16,	edgeFile = LSM:Fetch("border", dbCore.BorderTexture), edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 3}})
	fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	fraMinionAnchor.BorderFrame:Show()
	
	fraMinionAnchor.BottomFrame = SorhaQuestLog:doCreateFrame("FRAME","SQLActivitiesMinionBottom", fraMinionAnchor, db.MinionWidth,20,1,"BACKGROUND",1, "TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0, 1)
	
	blnMinionInitialized = true
	self:MinionAnchorUpdate(false)
end

function ActivitiesTracker:UpdateMinion()
	blnMinionUpdating = true
	
	if (blnMinionInitialized == false) then
		self:CreateMinionLayout()
	end	
	
	
	local parent = fraMinionAnchor:GetParent();
	if (parent) then
		if (parent:GetName() ~= db.MinionParent) then
			if (_G[db.MinionParent]) then
				fraMinionAnchor:SetParent(_G[db.MinionParent]);
			else 
				fraMinionAnchor:SetParent(UIParent);			
			end

			self:MinionAnchorUpdate(true);
		end
	end

	
	if (self:IsVisible() == false) then
		blnMinionUpdating = false
		return ""
	end
	
	-- Release all used buttons
	for k, objButton in pairs(tblUsingButtons) do
		self:RecycleMinionButton(objButton)
	end
	wipe(tblUsingButtons)

	self:UpdateData();


	local intYPosition = 4
	local intLargestWidth = 0
	local blnNothingShown = true
	
	-- Show title if enabled
	if (db.ShowTitle == true and (db.AutoHideTitle == false or (db.AutoHideTitle == true and activitiesData.ActivitiesCount > 0))) then
		blnNothingShown = false
		fraMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.MinionTitleFont), db.Fonts.MinionTitleFontSize, db.Fonts.MinionTitleFontOutline)
		if (db.Fonts.MinionTitleFontShadowed == true) then
			fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		fraMinionAnchor.objFontString:SetText(strMinionTitleColour .. L["Activities Tracker"] .. "|r");
		intLargestWidth = fraMinionAnchor.objFontString:GetWidth()
		
		intYPosition = db.Fonts.MinionTitleFontSize
	else
		fraMinionAnchor.objFontString:SetText("")
		intLargestWidth = 100
	end			
	fraMinionAnchor:SetWidth(db.MinionWidth)

	if (activitiesData.ActivitiesCount > 0) then
		blnNothingShown = false
	end
		
	-- Create main minions buttons and set text etc	
	local intInitialYOffset = intYPosition
	
	for k, ActivitiesInstance in pairs(activitiesData.ActivitiesList) do
		local objButton = ActivitiesTracker:GetMinionActivityButton(ActivitiesInstance)
		objButton:SetPoint("TOPLEFT", fraMinionAnchor.fraActivitiesAnchor, "TOPLEFT", 0, -intYPosition);
		if (objButton.Width > intLargestWidth) then
			intLargestWidth = objButton.Width
		end

		local totalHeight = objButton.Height;
		intYPosition = intYPosition + objButton.Height;

		local displayedCriteria = 0;
		
		if (ActivitiesInstance.IsCollapsed == false) then
			for i, RequirementInstance in spairs(ActivitiesInstance.RequirementsList, function(t,a,b) 
				return t[b].Text > t[a].Text 
			end) do
				if (RequirementInstance.IsComplete == false) then
					
					--Max count
					if (db.MaxRequirementsEachActivity > 0 and displayedCriteria >= db.MaxRequirementsEachActivity) then
						local objButtonCriteria = ActivitiesTracker:GetMinionRequirementCappedButton(RequirementInstance)
						objButtonCriteria:SetPoint("TOPLEFT", fraMinionAnchor.fraActivitiesAnchor, "TOPLEFT", db.ActivityContentIndent, -intYPosition);
						if (objButtonCriteria.Width > intLargestWidth) then
							intLargestWidth = objButtonCriteria.Width
						end
						intYPosition = intYPosition + objButtonCriteria.Height;	
						totalHeight = totalHeight + objButtonCriteria.Height;
						break;
					end


					local objButtonCriteria = ActivitiesTracker:GetMinionRequirementButton(RequirementInstance)
					objButtonCriteria:SetPoint("TOPLEFT", fraMinionAnchor.fraActivitiesAnchor, "TOPLEFT", db.ActivityContentIndent, -intYPosition);
					if (objButtonCriteria.Width > intLargestWidth) then
						intLargestWidth = objButtonCriteria.Width
					end
					
					intYPosition = intYPosition + objButtonCriteria.Height;	
					totalHeight = totalHeight + objButtonCriteria.Height;
					displayedCriteria = displayedCriteria +1
				end
			end
		end
		objButton:SetHeight(totalHeight);

		intYPosition = intYPosition + db.PaddingAfterActivity;
	end

	local intBorderWidth = db.MinionWidth
	-- Auto collapse
	
	if (db.MinionCollapseToLeft == true) then
		if (intLargestWidth < db.MinionWidth) then
			fraMinionAnchor:SetWidth(intLargestWidth)
			intBorderWidth = intLargestWidth
			
			for k, objButton in pairs(tblUsingButtons) do
				if (objButton.RequirementInstance ~= nil) then
					objButton.objFontString1:SetWidth(intLargestWidth - db.ActivityContentIndent)
					objButton:SetWidth(intLargestWidth - db.ActivityContentIndent)
				else
					objButton.objFontString1:SetWidth(intLargestWidth)
					objButton:SetWidth(intLargestWidth)
				end
			end
		end
	end
	
	
	-- Show border if at least the title is shown
	if (blnNothingShown == true) then
		fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, 0)
		fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, 0)		
	else
		fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
		fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)	
		fraMinionAnchor.BorderFrame:SetWidth(intBorderWidth + 16)
	end
	
	fraMinionAnchor.BorderFrame:ClearAllPoints()
	
	-- Reposition/Resize the border and the Activitiess Anchor based on grow upwards option
	if (db.GrowUpwards == false) then
		fraMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraMinionAnchor.fraActivitiesAnchor, "TOPLEFT", -8, 6);
		fraMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraMinionAnchor:GetHeight()/2)
		fraMinionAnchor.fraActivitiesAnchor:ClearAllPoints()
		fraMinionAnchor.fraActivitiesAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0);
	else
		fraMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraMinionAnchor.fraActivitiesAnchor, "TOPLEFT", -8,  6 - intInitialYOffset);
		fraMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraMinionAnchor:GetHeight()/2)
		fraMinionAnchor.fraActivitiesAnchor:ClearAllPoints()
		fraMinionAnchor.fraActivitiesAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, intYPosition);
	end

	fraMinionAnchor.BottomFrame:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, -intYPosition);
	fraMinionAnchor.BottomFrame:SetWidth(fraMinionAnchor:GetWidth());
	
	blnMinionUpdating = false
end 

function ActivitiesTracker:UpdateData()
	if (activitiesData == nil) then
		activitiesData = SQLActivitiesData:new();	
	else
		activitiesData:Update();
	end
end


--Uniform
function ActivitiesTracker:MinionAnchorUpdate(blnMoveAnchors)
	if (blnMinionInitialized == false) then
		if (self:IsVisible() == true) then
			self:CreateMinionLayout()
		end
	end	
	
	if (blnMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.MinionLocked == false) then
			fraMinionAnchor:EnableMouse(true)
		else
			fraMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Minion
		if (self:IsVisible() == true and dbCore.Main.HideAll == false) then
			fraMinionAnchor:Show()
			if (dbCore.Main.ShowAnchors == true and db.MinionLocked == false) then
				fraMinionAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraMinionAnchor:SetBackdropColor(0, 1, 0, 0)
			end
			
			if (blnMinionUpdating == false) then
				self:UpdateMinion()
			end
		else
			fraMinionAnchor:Hide()
		end

		fraMinionAnchor.BorderFrame:SetBackdrop({bgFile = LSM:Fetch("background", dbCore.BackgroundTexture), tile = false, tileSize = 16,	edgeFile = LSM:Fetch("border", dbCore.BorderTexture), edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 3}})
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraMinionAnchor:ClearAllPoints()
			local isUIParent = false;
			if (db.MinionParent == "UIParent" or fraMinionAnchor:GetParent() == UIParent) then
				isUIParent = true;
			end

			if (isUIParent == true) then
				fraMinionAnchor:SetPoint(db.MinionLocation.Point, UIParent, db.MinionLocation.RelativePoint, db.MinionLocation.X, db.MinionLocation.Y);
			else
				fraMinionAnchor:SetPoint("TOPLEFT", db.MinionParent,"TOPLEFT", 0, -4);
			end
			fraMinionAnchor:SetScale(db.MinionScale);
		end
	end
end

function ActivitiesTracker:UpdateColourStrings()
	strMinionTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.MinionTitleColour.r * 255, db.Colours.MinionTitleColour.g * 255, db.Colours.MinionTitleColour.b * 255);
	strActivityTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ActivityTitleColour.r * 255, db.Colours.ActivityTitleColour.g * 255, db.Colours.ActivityTitleColour.b * 255);
	strActivityDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ActivityDescriptionColour.r * 255, db.Colours.ActivityDescriptionColour.g * 255, db.Colours.ActivityDescriptionColour.b * 255);	
	strActivityRequirementColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ActivityRequirementColour.r * 255, db.Colours.ActivityRequirementColour.g * 255, db.Colours.ActivityRequirementColour.b * 255);
	strActivityReqirementUndoneColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ActivityRequirementUndoneColour.r * 255, db.Colours.ActivityRequirementUndoneColour.g * 255, db.Colours.ActivityRequirementUndoneColour.b * 255);
	
end

function ActivitiesTracker:HandleColourChanges()
	self:UpdateColourStrings()
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

function ActivitiesTracker:ToggleLockState()
	db.MinionLocked = not db.MinionLocked
end

function ActivitiesTracker:IsVisible()
	if (self:IsEnabled() == true and dbCore.Main.HideAll == false) then
		return true
	end
	return false	
end
