local SorhaQuestLog = LibStub("AceAddon-3.0"):GetAddon("SorhaQuestLog")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local MODNAME = "AdventureTracker"
local AdventureTracker = SorhaQuestLog:NewModule(MODNAME, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "LibSink-2.0")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")

local fraMinionAnchor = nil
local blnMinionInitialized = false
local blnMinionUpdating = false

local strButtonPrefix = MODNAME .. "Button"


local intNumberUsedButtons = 0
local tblButtonCache = {}
local tblUsingButtons = {}

local strMinionTitleColour = "|cffffffff"
local strAdventureItemTitleColour = "|cffffffff"
local strAdventureItemObjectiveColour = "|cffffffff"
local strAdventureItemObjectivePOIColour = "|cff33ff33"



local adventureData = nil;

local NavigableContentTrackingTargets = {
	[Enum.ContentTrackingTargetType.Vendor] = true,
	[Enum.ContentTrackingTargetType.JournalEncounter] = true,
};

local dicOutlines = {
	[""] = NONE,
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
	["MONOCHROMEOUTLINE"] = L["Monochrome Outline"],
}

--Defaults
local db
local dbCore
local dbChar;
local defaults = {
	profile = {
		MinionParent = "UIParent",
		MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
		MinionScale = 1,
		MinionLocked = false,
		MinionWidth = 220,
		MinionCollapseToLeft = false,
		MoveTooltipsRight = false,
		ShowTitle = true,
		AutoHideTitle = false,
		GrowUpwards = false,
		AdventureItemContentIndent = 5,
		PaddingAfterAdventureItem = 3,
		Fonts = {
			MinionTitleFontSize = 11,
			MinionTitleFont = "framd",
			MinionTitleFontOutline = "",
			MinionTitleFontShadowed = true,
			
			AdventureItemTitleFontSize = 11,
			AdventureItemTitleFont = "framd",
			AdventureItemTitleFontOutline = "",
			AdventureItemTitleFontShadowed = true,
						
			AdventureItemObjectiveFontSize = 11,
			AdventureItemObjectiveFont = "framd",
			AdventureItemObjectiveFontOutline = "",
			AdventureItemObjectiveFontShadowed = true,			
		},
		Colours = {
			MinionTitleColour = {r = 0, g = 1, b = 0, a = 1},
			AdventureItemTitleColour = {r = 0, g = 1, b = 0, a = 1},
			AdventureItemObjectiveColour = {r = 1, g = 1, b = 1, a = 1},			
			AdventureItemObjectivePOIColour = {r = 1, g = 1, b = 1, a = 1},		
			MinionBackGroundColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			MinionBorderColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
		}
	},
	char = {
		AdventureItemCollapsedState = {},
	}
}

-- Options
local options
local function getOptions()
	if not options then
		options = {
			name = L["Adventure Item Settings"],
			type = "group",
			childGroups = "tab",
			order = 2,
			arg = MODNAME,
			args = {
				Main = {
					name = "Main",
					type = "group",
					order = 1,
					args = {
						enabled = {
							order = 1,
							type = "toggle",
							name = L["Enable Minion"],
							get = function() return SorhaQuestLog:GetModuleEnabled(MODNAME) end,
							set = function(info, value) 
								SorhaQuestLog:SetModuleEnabled(MODNAME, value) 
								AdventureTracker:MinionAnchorUpdate(false)
							end,
						},
						MinionLockedToggle = {
							name = L["Lock Minion"],
							type = "toggle",
							get = function() return db.MinionLocked end,
							set = function()
								db.MinionLocked = not db.MinionLocked
								AdventureTracker:MinionAnchorUpdate(false)
							end,
							order = 2,
						},
						ShowTitleToggle = {
							name = L["Show Minion Title"],
							type = "toggle",
							get = function() return db.ShowTitle end,
							set = function()
								db.ShowTitle = not db.ShowTitle
								AdventureTracker:UpdateMinion()
							end,
							order = 3,
						},
						AutoHideTitleToggle = {
							name = L["Auto Hide Minion Title"],
							desc = L["Hide the title when there is nothing to display"],
							type = "toggle",
							disabled = function() return not(db.ShowTitle) end,
							get = function() return db.AutoHideTitle end,
							set = function()
								db.AutoHideTitle = not db.AutoHideTitle
								AdventureTracker:UpdateMinion(false)
							end,
							order = 4,
						},
						GrowUpwardsToggle = {
							name = L["Grow Upwards"],
							desc = L["Minions grows upwards from the anchor"],
							type = "toggle",
							get = function() return db.GrowUpwards end,
							set = function()
								db.GrowUpwards = not db.GrowUpwards
								AdventureTracker:UpdateMinion()
							end,
							order = 5,
						},						
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current content is less then the max width\nNote: Doesn't work well with content that word wraps"],
							type = "toggle",
							get = function() return db.MinionCollapseToLeft end,
							set = function()
								db.MinionCollapseToLeft = not db.MinionCollapseToLeft
								AdventureTracker:UpdateMinion()
							end,
							order = 86,
						},
						MoveTooltipsRightToggle = {
							name = L["Tooltips on right"],
							desc = L["Moves the tooltips to the right"],
							type = "toggle",
							get = function() return db.MoveTooltipsRight end,
							set = function()
								db.MoveTooltipsRight = not db.MoveTooltipsRight
								AdventureTracker:UpdateMinion()
							end,
							order = 7,
						},
						MinionScaler = {
							order = 10,
							name = L["Minion Scale"],
							desc = L["Adjust the scale of the minion"],
							type = "range",
							min = 0.5, max = 2, step = 0.05,
							isPercent = false,
							get = function() return db.MinionScale end,
							set = function(info, value)
								db.MinionScale = value
								AdventureTracker:MinionAnchorUpdate(true)
							end,
						},
						MinionWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Adjust the width of the minion"],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.MinionWidth end,
							set = function(info, value)
								db.MinionWidth = value
								AdventureTracker:UpdateMinion()
							end,
						},		
						MinionParent = {
							name = L["Minion Anchor Point"],
							desc = L["The minion to anchor this minion to"],
							type = "select",
							order = 12,
							values = function() return SorhaQuestLog:GetPossibleParents(fraMinionAnchor)end,
							get = function() return db.MinionParent  end,
							set = function(info, value)
								db.MinionParent = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureMinionReset = {
							order = 14,
							type = "execute",
							name = L["Reset Minion Position"],
							desc = L["Resets Adventure Item Minions position"],
							func = function()
								db.MinionLocation.Point = "CENTER"
								db.MinionLocation.RelativePoint =  "CENTER"
								db.MinionLocation.X = 0
								db.MinionLocation.Y = 0
								AdventureTracker:MinionAnchorUpdate(true)
							end,
						},
						AdventureSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 30,
						},
						AdventureSpacerHeader = {
							name = L["Adventure Item Settings"],
							type = "header",
							order = 31,
						},
						ContentIndent = {
							order = 35,
							name = L["Objective Indent"],
							desc = L["Controls the level of indentation for the Adventure items objective"],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.AdventureItemContentIndent end,
							set = function(info, value)
								db.AdventureItemContentIndent = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemAfterPadding = {
							order = 36,
							name = L["Adventure Item Padding"],
							desc = L["The amount of extra padding after an Adventure Item before the next."],
							type = "range",
							min = 0, max = 20, step = 1,
							isPercent = false,
							get = function() return db.PaddingAfterAdventureItem end,
							set = function(info, value)
								db.PaddingAfterAdventureItem = value
								AdventureTracker:UpdateMinion()
							end,
						}
					}		
				},
				Fonts = {
					name = "Fonts",
					type = "group",
					order = 5,
					args = {
						HeaderTitleFont = {
							name = L["Minion Title Font Settings"],
							type = "header",
							order = 41,
						},
						MinionTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 42,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.MinionTitleFont end,
							set = function(info, value)
								db.Fonts.MinionTitleFont = value
								AdventureTracker:UpdateMinion()
							end,
						},
						MinionTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 43,
							values = dicOutlines,
							get = function() return db.Fonts.MinionTitleFontOutline end,
							set = function(info, value)
								db.Fonts.MinionTitleFontOutline = value
								AdventureTracker:UpdateMinion()
							end,
						},
						MinionTitleFontSizeSelect = {
							order = 44,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.MinionTitleFontSize end,
							set = function(info, value)
								db.Fonts.MinionTitleFontSize = value
								AdventureTracker:UpdateMinion()
							end,
						},
						MinionTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.MinionTitleFontShadowed end,
							set = function()
								db.Fonts.MinionTitleFontShadowed = not db.Fonts.MinionTitleFontShadowed
								AdventureTracker:UpdateMinion()
							end,
							order = 45,
						},
						
						HeaderFontsSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderFonts = {
							name = L["Adventure Item Font Settings"],
							type = "header",
							order = 51,
						},
						AdventureItemTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 52,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AdventureItemTitleFont end,
							set = function(info, value)
								db.Fonts.AdventureItemTitleFont = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemTitleFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 53,
							values = dicOutlines,
							get = function() return db.Fonts.AdventureItemTitleFontOutline end,
							set = function(info, value)
								db.Fonts.AdventureItemTitleFontOutline = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemTitleFontSizeSize = {
							order = 54,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AdventureItemTitleFontSize end,
							set = function(info, value)
								db.Fonts.AdventureItemTitleFontSize = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemTitleFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.AdventureItemTitleFontShadowed end,
							set = function()
								db.Fonts.AdventureItemTitleFontShadowed = not db.Fonts.AdventureItemTitleFontShadowed
								AdventureTracker:UpdateMinion()
							end,
							order = 55,
						},
						
						RequirementFontsSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 60,
						},
						RequirementFonts = {
							name = L["Objective Font Settings"],
							type = "header",
							order = 61,
						},
						AdventureItemObjectiveFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 62,
							name = L["Font"],
							desc = L["The font used for this element"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AdventureItemObjectiveFont end,
							set = function(info, value)
								db.Fonts.AdventureItemObjectiveFont = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemObjectiveFontOutlineSelect = {
							name = L["Font Outline"],
							desc = L["The outline that this font will use"],
							type = "select",
							order = 63,
							values = dicOutlines,
							get = function() return db.Fonts.AdventureItemObjectiveFontOutline end,
							set = function(info, value)
								db.Fonts.AdventureItemObjectiveFontOutline = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemObjectiveFontSize = {
							order = 64,
							name = FONT_SIZE,
							desc = L["Controls the font size this font"],
							type = "range",
							min = 8, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AdventureItemObjectiveFontSize end,
							set = function(info, value)
								db.Fonts.AdventureItemObjectiveFontSize = value
								AdventureTracker:UpdateMinion()
							end,
						},
						AdventureItemObjectiveFontShadowedToggle = {
							name = L["Shadow Text"],
							desc = L["Shows/Hides text shadowing"],
							type = "toggle",
							get = function() return db.Fonts.AdventureItemObjectiveFontShadowed end,
							set = function()
								db.Fonts.AdventureItemObjectiveFontShadowed = not db.Fonts.AdventureItemObjectiveFontShadowed
								AdventureTracker:UpdateMinion()
							end,
							order = 65,
						},
					}
				},
				Colours = {
					name = "Colours",
					type = "group",
					order = 6,
					args = {
						HeaderColourSettings = {
							name = L["Colour Settings"],
							type = "header",
							order = 80,
						},
						MinionTitleColour = {
							name = L["Minion Title"],
							desc = L["Sets the color for Minion Title"],
							type = "color",
							width = "full",
							hasAlpha = true,
							get = function() return db.Colours.MinionTitleColour.r, db.Colours.MinionTitleColour.g, db.Colours.MinionTitleColour.b, db.Colours.MinionTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionTitleColour.r = r
									db.Colours.MinionTitleColour.g = g
									db.Colours.MinionTitleColour.b = b
									db.Colours.MinionTitleColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 81,
						},
						AdventureItemTitleColour = {
							name = L["Adventure Item Titles"],
							desc = L["Sets the color for Adventure Item Titles"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.AdventureItemTitleColour.r, db.Colours.AdventureItemTitleColour.g, db.Colours.AdventureItemTitleColour.b, db.Colours.AdventureItemTitleColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.AdventureItemTitleColour.r = r
									db.Colours.AdventureItemTitleColour.g = g
									db.Colours.AdventureItemTitleColour.b = b
									db.Colours.AdventureItemTitleColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 82,
						},
						AdventureItemObjectiveColour = {
							name = L["Adventure Item Objectives"],
							desc = L["Sets the color for Adventure Item Objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.AdventureItemObjectiveColour.r, db.Colours.AdventureItemObjectiveColour.g, db.Colours.AdventureItemObjectiveColour.b, db.Colours.AdventureItemObjectiveColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.AdventureItemObjectiveColour.r = r
									db.Colours.AdventureItemObjectiveColour.g = g
									db.Colours.AdventureItemObjectiveColour.b = b
									db.Colours.AdventureItemObjectiveColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 84,
						},
						AdventureItemObjectivePOIColourSelector = {
							name = L["Adventure Item POI Hint"],
							desc = L["Sets the color for Adventure Item POI Hints"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.AdventureItemObjectivePOIColour.r, db.Colours.AdventureItemObjectivePOIColour.g, db.Colours.AdventureItemObjectivePOIColour.b, db.Colours.AdventureItemObjectivePOIColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.AdventureItemObjectivePOIColour.r = r
									db.Colours.AdventureItemObjectivePOIColour.g = g
									db.Colours.AdventureItemObjectivePOIColour.b = b
									db.Colours.AdventureItemObjectivePOIColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 85,
						},
						MinionBackGroundColour = {
							name = L["Background Colour"],
							desc = L["Sets the color of the minions background"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionBackGroundColour.r = r
									db.Colours.MinionBackGroundColour.g = g
									db.Colours.MinionBackGroundColour.b = b
									db.Colours.MinionBackGroundColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 89,
						},
						MinionBorderColour = {
							name = L["Border Colour"],
							desc = L["Sets the color of the minions border"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.MinionBorderColour.r = r
									db.Colours.MinionBorderColour.g = g
									db.Colours.MinionBorderColour.b = b
									db.Colours.MinionBorderColour.a = a
									AdventureTracker:HandleColourChanges()
								end,
							order = 90,
						},
					}
				},
			}
		}
	end

	return options
end


local function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

local SQLAdventureItemObjective = {};
SQLAdventureItemObjective.__index = SQLAdventureItemObjective; 

function SQLAdventureItemObjective:new(objectiveKey, type, id) 
	local self = {};
	setmetatable(self, SQLAdventureItemObjective);

	self.Index = objectiveKey;
	self.Id = id;
	
	self.TargetType = nil;
	self.TargetId = nil;

	self.Type = type;
	self.Text = ""

	self.POIText = "";

	self.IsComplete = false;
	self.Changed = false;

	self:Update();

	return self;
end

function SQLAdventureItemObjective:Update()
	self.TargetType, self.TargetId = C_ContentTracking.GetCurrentTrackingTarget(self.Type, self.Id);
	local text = C_ContentTracking.GetObjectiveText(self.TargetType, self.TargetId);
	if (text == nil) then
		text = "";
	end
	self.Changed = false;
	if (self.Text ~= text) then
		self.Changed = true;
	end
	self.Text = text;
	
	local lastPOIText = self.POIText;
	self.POIText = "";

	if NavigableContentTrackingTargets[self.TargetType] then
		local trackingResult, uiMapID = C_ContentTracking.GetBestMapForTrackable(self.Type, self.Id, true);
		if (trackingResult ~= Enum.ContentTrackingResult.DataPending) then

			local navigableTrackingResult, isNavigable = C_ContentTracking.IsNavigable(self.Type, self.Id);
			local superTrackedType, superTrackedID = C_SuperTrack.GetSuperTrackedContent();
			if (self.Type == superTrackedType) and (self.Id == superTrackedID) then
				local waypointText = C_ContentTracking.GetWaypointText(self.Type, self.Id);

				if (waypointText ~= nil) then
					self.POIText = waypointText;
				end
			end
		end

	end

	
	if(self.POIText ~= lastPOIText) then
		self.Changed = true;
	end
end

function SQLAdventureItemObjective:Render()
	local strOutput = strAdventureItemObjectiveColour;
	strOutput = strOutput ..  self.Text .. "|r";	
	return strOutput;
end

function SQLAdventureItemObjective:RenderPOI()
	if (self.POIText ~= "") then
		local strOutput = strAdventureItemObjectivePOIColour;
		strOutput = strOutput .. "- " .. self.POIText .. "|r";	
		return strOutput;
	end

	return "";
end

local SQLAdventureItem = {};
SQLAdventureItem.__index = SQLAdventureItem; 

function SQLAdventureItem:new(id) 
	local self = {};
	setmetatable(self, SQLAdventureItem);

	self.Type = Enum.ContentTrackingType.Appearance;
	self.Id = id;
	self.Name = "";

	self.ObjectivesList = {};
	self.ObjectivesCount = 0;

	self.Changed = false;
	self.Keep = false;

	self.IsCollapsed = false;
	if (dbChar.AdventureItemCollapsedState[self.Id] == true) then
		self.IsCollapsed = true;
	end

	self:Update();

	return self;
end

function SQLAdventureItem:Update()
	self.Changed = false;
	self.Keep = true;

	self.Name = C_ContentTracking.GetTitle(self.Type, self.Id);
	self.Keep = C_ContentTracking.IsTracking(self.Type, self.Id);
	if (self.Name == nil) then
		self.Name = "";
	end

	--Only one thing for now
	-- for objectiveKey, objective in pairs(activityInfo.requirementsList) do
	-- 	if (self.ObjectivesList[objectiveKey] == nil) then
	-- 		self.ObjectivesList[objectiveKey] = SQLAdventureItemObjective:new(objectiveKey, self.Type, self.Id);
	-- 		self.ObjectivesCount = self.ObjectivesCount + 1;
	-- 	else
	-- 		self.ObjectivesList[objectiveKey]:Update(objective);
	-- 	end
	-- end
	if (self.ObjectivesList[1] == nil) then
		self.ObjectivesList[1] = SQLAdventureItemObjective:new(1, self.Type, self.Id);
		self.ObjectivesCount = self.ObjectivesCount + 1;
	else
		self.ObjectivesList[1]:Update(objective);
	end
	--If any requirement changed, then changed

	return self;
end

function SQLAdventureItem:Render()
	return strAdventureItemTitleColour .. self.Name .. "|r";
end


local SQLAdventureData = {};
SQLAdventureData.__index = SQLAdventureData; 


function SQLAdventureData:new() 
	local self = {};
	setmetatable(self, SQLAdventureData);

	self.FirstUpdate = true;
	self.EntryCount = 0;

	self.ItemCount = 0;
	self.ItemList = {};

	self.Changed = false;
	self:Update();
	return self;
end

function SQLAdventureData:Update()
	--self.Changed = false;

	for itemKey, item in pairs(self.ItemList) do
    	item.Keep = false;
	end

	
	local trackedIDs = C_ContentTracking.GetTrackedIDs(Enum.ContentTrackingType.Appearance);
	for i = 1, #trackedIDs  do
		local trackedID = trackedIDs[i];
		if (self.ItemList[trackedID] == nil) then
			self.ItemList[trackedID] = SQLAdventureItem:new(trackedID);
			self.ItemCount = self.ItemCount + 1;
		else
			self.ItemList[trackedID]:Update();
		end
	end

	for itemKey, item in pairs(self.ItemList) do
    	if (item.Keep == false) then
    		self.ItemList[item.Id] = nil;
    		self.ItemCount = self.ItemCount - 1;
    	end
	end
	self.FirstUpdate = false;
end

--Inits
function AdventureTracker:OnInitialize()
	self.db = SorhaQuestLog.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	dbChar = self.db.char
	dbCore = SorhaQuestLog.db.profile
	self:SetEnabledState(SorhaQuestLog:GetModuleEnabled(MODNAME))
	SorhaQuestLog:RegisterModuleOptions(MODNAME, getOptions, L["Adventure Item Tracker"])
	
	self:UpdateColourStrings()
	self:MinionAnchorUpdate(true)
end

function AdventureTracker:OnEnable()
	self:RegisterEvent("CONTENT_TRACKING_UPDATE");
	self:RegisterEvent("TRANSMOG_COLLECTION_SOURCE_ADDED");
	self:RegisterEvent("SUPER_TRACKING_CHANGED");
	
	self:MinionAnchorUpdate(false)
	self:UpdateMinion()
end

function AdventureTracker:OnDisable()
	self:UnregisterEvent("CONTENT_TRACKING_UPDATE")	
	self:UnregisterEvent("TRANSMOG_COLLECTION_SOURCE_ADDED")	
	self:UnregisterEvent("SUPER_TRACKING_CHANGED")	
	self:MinionAnchorUpdate(true)
	self:UpdateMinion()
end

function AdventureTracker:Refresh()
	db = self.db.profile
	dbCore = SorhaQuestLog.db.profile
	
	self:HandleColourChanges()
	self:MinionAnchorUpdate(true)
end

--Events/handlers
function AdventureTracker:CONTENT_TRACKING_UPDATE(...)	
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then 
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

function AdventureTracker:TRANSMOG_COLLECTION_SOURCE_ADDED(...)
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then 
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

function AdventureTracker:SUPER_TRACKING_CHANGED(...)	
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then 
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

--Buttons
function AdventureTracker:GetMinionButton()
	local objButton = SorhaQuestLog:GetLogButton()
	objButton:SetParent(fraMinionAnchor)
	tinsert(tblUsingButtons, objButton);
	
	objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);

	return objButton
end


function AdventureTracker:GetMinionAdventureItemButton(itemInstance)
	local objButton = AdventureTracker:GetMinionButton(itemInstance)
	objButton.AdventureItemInstance = itemInstance;

	-- Set buttons title text
	objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.AdventureItemTitleFont), db.Fonts.AdventureItemTitleFontSize, db.Fonts.AdventureItemTitleFontOutline)
	if (db.Fonts.AdventureItemTitleFontShadowed == true) then
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	objButton.objFontString1:SetText(objButton.AdventureItemInstance:Render())
	if (objButton.objFontString1:GetWidth() > db.MinionWidth) then
		objButton.objFontString1:SetWidth(db.MinionWidth)
	end

	
	local currentHeight = 0;
	local currentWidth = 0;

	currentWidth = objButton.objFontString1:GetWidth();

	if (db.Fonts.AdventureItemTitleFontOutline == "THICKOUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 2;
	elseif (db.Fonts.AdventureItemTitleFontOutline == "OUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 1;
	else
		currentHeight = objButton.objFontString1:GetHeight();
	end
	

	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		if (button == "LeftButton") then
			if (IsControlKeyDown()) then
				C_SuperTrack.SetSuperTrackedContent(self.AdventureItemInstance.Type, self.AdventureItemInstance.Id)

			elseif (IsShiftKeyDown()) then
				DressUpVisual(self.AdventureItemInstance.Id);

			else
				TransmogUtil.OpenCollectionToItem(self.AdventureItemInstance.Id);
			end
		else
			if (IsControlKeyDown()) then
				C_ContentTracking.StopTracking(self.AdventureItemInstance.Type, self.AdventureItemInstance.Id, Enum.ContentTrackingStopType.Manual);
			else
				self.AdventureItemInstance.IsCollapsed = not self.AdventureItemInstance.IsCollapsed;
				dbChar.AdventureItemCollapsedState[self.AdventureItemInstance.Id] = self.AdventureItemInstance.IsCollapsed;
				AdventureTracker:UpdateMinion()
			end
		end
	end)

	objButton:SetScript("OnEnter", function(self)
		if (db.MoveTooltipsRight == true) then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
		else 
			GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
		end

		GameTooltip:SetText(self.AdventureItemInstance:Render(), 0, 1, 0, 1);
		
		if (self.AdventureItemInstance.ObjectivesCount > 0) then
			GameTooltip:AddLine(" ", 0.5, 0.5, 0.5, 1);
		end
		for i, criteria in spairs(self.AdventureItemInstance.ObjectivesList, function(t,a,b) return t[b].Text > t[a].Text end) do
			GameTooltip:AddLine(criteria:Render(), 0.5, 0.5, 0.5, 1);
			if (criteria:RenderPOI() ~= "") then
				GameTooltip:AddLine(" ", 0.5, 0.5, 0.5, 1);
				GameTooltip:AddLine(criteria:RenderPOI(), 0.5, 0.5, 0.5, 1);
			end
		end

		if (dbCore.Main.ShowHelpTooltips == true) then
			GameTooltip:AddLine(" ", 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Click Interactions"], 1, 1, 1, 1);
			GameTooltip:AddLine(L["Left Mouse"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Normal: Open to Collection"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Ctrl: Set as tracked"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Shift: View on Character"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(" ", 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Right Mouse"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Normal: Collapse/Expand"], 0.5, 0.5, 0.5, 1);
			GameTooltip:AddLine(L["Ctrl: Untrack"], 0.5, 0.5, 0.5, 1);
		end
		GameTooltip:Show();
	end)

	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)

	objButton.Width = currentWidth;
	objButton.Height = currentHeight;
	objButton:SetWidth(db.MinionWidth);	
	objButton:SetScript("OnHyperlinkClick", function(self, a, b, c) 
		print('rawr')
		SetItemRef(a, b, c);
	end)
	return objButton
end

function AdventureTracker:GetMinionObjectiveButton(objectiveInstance)
	local objButton = AdventureTracker:GetMinionButton(objectiveInstance)
	objButton.ObjectiveInstance = objectiveInstance;

	objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.AdventureItemObjectiveFont), db.Fonts.AdventureItemObjectiveFontSize, db.Fonts.AdventureItemObjectiveFontOutline)
	if (db.Fonts.AdventureItemObjectiveFontShadowed == true) then
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	local currentHeight = 0;
	local currentWidth = 0;

	objButton.objFontString1:SetText(objButton.ObjectiveInstance:Render());

	if (objButton.objFontString1:GetWidth() > (db.MinionWidth - db.AdventureItemContentIndent)) then
		objButton.objFontString1:SetWidth(db.MinionWidth - db.AdventureItemContentIndent)
	end

	currentWidth = objButton.objFontString1:GetWidth();

	if (db.Fonts.AdventureItemObjectiveFontOutline == "THICKOUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 1.5;
	elseif (db.Fonts.AdventureItemObjectiveFontOutline == "OUTLINE") then
		currentHeight = objButton.objFontString1:GetHeight() + 0.5;
	else
		currentHeight = objButton.objFontString1:GetHeight();
	end	
	
	local poiText = objButton.ObjectiveInstance:RenderPOI();
	if (poiText ~= "") then
		objButton.objFontString2:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, -currentHeight);
		objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.AdventureItemObjectiveFont), db.Fonts.AdventureItemObjectiveFontSize, db.Fonts.AdventureItemObjectiveFontOutline)
		if (db.Fonts.AdventureItemObjectiveFontShadowed == true) then
			objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end	
		objButton.objFontString2:SetText(poiText);



		if (objButton.objFontString2:GetWidth() > (db.MinionWidth - db.AdventureItemContentIndent)) then
			objButton.objFontString2:SetWidth(db.MinionWidth - db.AdventureItemContentIndent)
		end

		if (objButton.objFontString2:GetWidth() > currentWidth) then
			currentWidth = objButton.objFontString2:GetWidth();
		end
		currentHeight = currentHeight + objButton.objFontString2:GetHeight()
	else

	end

	--objButton:EnableMouse(true)
	objButton:SetHyperlinksEnabled(true)

	objButton:SetScript("OnHyperlinkClick", function(self, link, text, button) 
		SetItemRef(link, text, button);
	end)


	objButton.Width = currentWidth;
	objButton.Height = currentHeight;

	objButton:SetHeight(objButton.Height);
	objButton:SetWidth(db.MinionWidth);
	return objButton
end

function AdventureTracker:RecycleMinionButton(objButton)
	objButton.Height = 0;
	objButton.AdventureItemInstance = nil;
	objButton.ObjectiveInstance = nil;

	--objButton:EnableMouse(true)
	objButton:SetHyperlinksEnabled(false)
	objButton:SetScript("OnHyperlinkClick", nil)	

	SorhaQuestLog:RecycleLogButton(objButton);
end


--Minion
function AdventureTracker:CreateMinionLayout()
	fraMinionAnchor = SorhaQuestLog:doCreateFrame("FRAME","SQLAdventureMinionAnchor",UIParent,100,20,1,"BACKGROUND",1, db.MinionLocation.Point, UIParent, db.MinionLocation.RelativePoint, db.MinionLocation.X, db.MinionLocation.Y, 1, "BackdropTemplate")
	fraMinionAnchor:SetMovable(true)
	fraMinionAnchor:SetClampedToScreen(true)
	fraMinionAnchor:RegisterForDrag("LeftButton")
	fraMinionAnchor:SetScript("OnDragStart", fraMinionAnchor.StartMoving)
	fraMinionAnchor:SetScript("OnDragStop",  function(self)
		fraMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraMinionAnchor:GetPoint()
		db.MinionLocation.Point = strPoint
		db.MinionLocation.RelativePoint = strRelativePoint
		db.MinionLocation.X = intPosX
		db.MinionLocation.Y = intPosY
	end)
	fraMinionAnchor:SetScript("OnEnter", function(self) 
		if (dbCore.Main.ShowHelpTooltips == true) then
			if (db.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, 0);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, 0);
			end
			
			GameTooltip:SetText(L["Adventure Item Minion Anchor"], 0, 1, 0, 1);
			GameTooltip:AddLine(L["Drag this to move the Adventure Item minion when it is unlocked."], 1, 1, 1, 1);
			GameTooltip:AddLine(L["You can disable help tooltips in general settings"], 0.5, 0.5, 0.5, 1);
			
			GameTooltip:Show();
		end
	end)
	fraMinionAnchor:SetScript("OnLeave", function(self) 
		GameTooltip:Hide()
	end)
	
	
	fraMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraMinionAnchor:SetBackdropColor(0, 0, 1, 0)
	fraMinionAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	
	-- Anchor
	fraMinionAnchor.fraMinionAnchor = SorhaQuestLog:doCreateLooseFrame("FRAME","SQLQuestsAnchor",fraMinionAnchor, fraMinionAnchor:GetWidth(),1,1,"LOW",1,1,"BackdropTemplate")
	fraMinionAnchor.fraMinionAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0);
	fraMinionAnchor.fraMinionAnchor:SetBackdropColor(0, 0, 0, 0)
	fraMinionAnchor.fraMinionAnchor:SetBackdropBorderColor(0,0,0,0)
	fraMinionAnchor.fraMinionAnchor:SetAlpha(0)
	
	
	-- Title Fontstring
	fraMinionAnchor.objFontString = fraMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraMinionAnchor.objFontString:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT",0, 0);
	fraMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.MinionTitleFont), db.Fonts.MinionTitleFontSize, db.Fonts.MinionTitleFontOutline)
	if (db.Fonts.MinionTitleFontShadowed == true) then
		fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	else
		fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
	end
	
	fraMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraMinionAnchor.objFontString:SetJustifyV("TOP")
	fraMinionAnchor.objFontString:SetText("");
	fraMinionAnchor.objFontString:SetShadowOffset(1, -1)
	
	fraMinionAnchor.BorderFrame = SorhaQuestLog:doCreateFrame("FRAME","SQLAdventureMinionBorder", fraMinionAnchor, 100,20,1,"BACKGROUND",1, "TOPLEFT", fraMinionAnchor, "TOPLEFT", -8, 6, 1, "BackdropTemplate")
	fraMinionAnchor.BorderFrame:SetBackdrop({bgFile = LSM:Fetch("background", dbCore.BackgroundTexture), tile = false, tileSize = 16,	edgeFile = LSM:Fetch("border", dbCore.BorderTexture), edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 3}})
	fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	fraMinionAnchor.BorderFrame:Show()
	
	fraMinionAnchor.BottomFrame = SorhaQuestLog:doCreateFrame("FRAME","SQLAdventureMinionBottom", fraMinionAnchor, db.MinionWidth,20,1,"BACKGROUND",1, "TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0, 1)
	
	blnMinionInitialized = true
	self:MinionAnchorUpdate(false)
end

function AdventureTracker:UpdateMinion()
	blnMinionUpdating = true
	
	if (blnMinionInitialized == false) then
		self:CreateMinionLayout()
	end	
	
	
	local parent = fraMinionAnchor:GetParent();
	if (parent) then
		if (parent:GetName() ~= db.MinionParent) then
			if (_G[db.MinionParent]) then
				fraMinionAnchor:SetParent(_G[db.MinionParent]);
			else 
				fraMinionAnchor:SetParent(UIParent);			
			end

			self:MinionAnchorUpdate(true);
		end
	end

	
	if (self:IsVisible() == false) then
		blnMinionUpdating = false
		return ""
	end
	
	-- Release all used buttons
	for k, objButton in pairs(tblUsingButtons) do
		self:RecycleMinionButton(objButton)
	end
	wipe(tblUsingButtons)

	self:UpdateData();


	local intYPosition = 4
	local intLargestWidth = 0
	local blnNothingShown = true
	
	-- Show title if enabled
	if (db.ShowTitle == true and (db.AutoHideTitle == false or (db.AutoHideTitle == true and adventureData.ItemCount > 0))) then
		blnNothingShown = false
		fraMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.MinionTitleFont), db.Fonts.MinionTitleFontSize, db.Fonts.MinionTitleFontOutline)
		if (db.Fonts.MinionTitleFontShadowed == true) then
			fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		else
			fraMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 0.0)
		end
		
		fraMinionAnchor.objFontString:SetText(strMinionTitleColour .. L["Adventure Item Tracker"] .. "|r");
		intLargestWidth = fraMinionAnchor.objFontString:GetWidth()
		
		intYPosition = db.Fonts.MinionTitleFontSize
	else
		fraMinionAnchor.objFontString:SetText("")
		intLargestWidth = 100
	end			
	fraMinionAnchor:SetWidth(db.MinionWidth)

	if (adventureData.ItemCount > 0) then
		blnNothingShown = false
	end
		
	-- Create main minions buttons and set text etc	
	local intInitialYOffset = intYPosition
	
	for k, itemInstance in pairs(adventureData.ItemList) do
		local objButton = AdventureTracker:GetMinionAdventureItemButton(itemInstance)
		objButton:SetPoint("TOPLEFT", fraMinionAnchor.fraMinionAnchor, "TOPLEFT", 0, -intYPosition);
		if (objButton.Width > intLargestWidth) then
			intLargestWidth = objButton.Width
		end

		local totalHeight = objButton.Height;
		intYPosition = intYPosition + objButton.Height;

		local displayedCriteria = 0;
		
		if (itemInstance.IsCollapsed == false) then
			for i, RequirementInstance in spairs(itemInstance.ObjectivesList, function(t,a,b) 
				return t[b].Text > t[a].Text 
			end) do
				if (RequirementInstance.IsComplete == false) then
					local objButtonCriteria = AdventureTracker:GetMinionObjectiveButton(RequirementInstance)
					objButtonCriteria:SetPoint("TOPLEFT", fraMinionAnchor.fraMinionAnchor, "TOPLEFT", db.AdventureItemContentIndent, -intYPosition);
					if (objButtonCriteria.Width > intLargestWidth) then
						intLargestWidth = objButtonCriteria.Width
					end
					
					intYPosition = intYPosition + objButtonCriteria.Height;	
					totalHeight = totalHeight + objButtonCriteria.Height;
					displayedCriteria = displayedCriteria +1
				end
			end
		end
		objButton:SetHeight(totalHeight);

		intYPosition = intYPosition + db.PaddingAfterAdventureItem;
	end

	local intBorderWidth = db.MinionWidth
	-- Auto collapse
	
	if (db.MinionCollapseToLeft == true) then
		if (intLargestWidth < db.MinionWidth) then
			fraMinionAnchor:SetWidth(intLargestWidth)
			intBorderWidth = intLargestWidth
			
			for k, objButton in pairs(tblUsingButtons) do
				if (objButton.RequirementInstance ~= nil) then
					objButton.objFontString1:SetWidth(intLargestWidth - db.AdventureItemContentIndent)
					objButton:SetWidth(intLargestWidth - db.AdventureItemContentIndent)
				else
					objButton.objFontString1:SetWidth(intLargestWidth)
					objButton:SetWidth(intLargestWidth)
				end
			end
		end
	end
	
	
	-- Show border if at least the title is shown
	if (blnNothingShown == true) then
		fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, 0)
		fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, 0)		
	else
		fraMinionAnchor.BorderFrame:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
		fraMinionAnchor.BorderFrame:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)	
		fraMinionAnchor.BorderFrame:SetWidth(intBorderWidth + 16)
	end
	
	fraMinionAnchor.BorderFrame:ClearAllPoints()
	
	-- Reposition/Resize the border and the Adventure Anchor based on grow upwards option
	if (db.GrowUpwards == false) then
		fraMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraMinionAnchor.fraMinionAnchor, "TOPLEFT", -8, 6);
		fraMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraMinionAnchor:GetHeight()/2)
		fraMinionAnchor.fraMinionAnchor:ClearAllPoints()
		fraMinionAnchor.fraMinionAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, 0);
	else
		fraMinionAnchor.BorderFrame:SetPoint("TOPLEFT", fraMinionAnchor.fraMinionAnchor, "TOPLEFT", -8,  6 - intInitialYOffset);
		fraMinionAnchor.BorderFrame:SetHeight(intYPosition + 2 + fraMinionAnchor:GetHeight()/2)
		fraMinionAnchor.fraMinionAnchor:ClearAllPoints()
		fraMinionAnchor.fraMinionAnchor:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, intYPosition);
	end

	fraMinionAnchor.BottomFrame:SetPoint("TOPLEFT", fraMinionAnchor, "TOPLEFT", 0, -intYPosition);
	fraMinionAnchor.BottomFrame:SetWidth(fraMinionAnchor:GetWidth());
	
	blnMinionUpdating = false
end 

function AdventureTracker:UpdateData()
	if (adventureData == nil) then
		adventureData = SQLAdventureData:new();	
	else
		adventureData:Update();
	end
end


--Uniform
function AdventureTracker:MinionAnchorUpdate(blnMoveAnchors)
	if (blnMinionInitialized == false) then
		if (self:IsVisible() == true) then
			self:CreateMinionLayout()
		end
	end	
	
	if (blnMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.MinionLocked == false) then
			fraMinionAnchor:EnableMouse(true)
		else
			fraMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Minion
		if (self:IsVisible() == true and dbCore.Main.HideAll == false) then
			fraMinionAnchor:Show()
			if (dbCore.Main.ShowAnchors == true and db.MinionLocked == false) then
				fraMinionAnchor:SetBackdropColor(0, 1, 0, 1)
			else
				fraMinionAnchor:SetBackdropColor(0, 1, 0, 0)
			end
			
			if (blnMinionUpdating == false) then
				self:UpdateMinion()
			end
		else
			fraMinionAnchor:Hide()
		end

		fraMinionAnchor.BorderFrame:SetBackdrop({bgFile = LSM:Fetch("background", dbCore.BackgroundTexture), tile = false, tileSize = 16,	edgeFile = LSM:Fetch("border", dbCore.BorderTexture), edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 3}})
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraMinionAnchor:ClearAllPoints()
			local isUIParent = false;
			if (db.MinionParent == "UIParent" or fraMinionAnchor:GetParent() == UIParent) then
				isUIParent = true;
			end

			if (isUIParent == true) then
				fraMinionAnchor:SetPoint(db.MinionLocation.Point, UIParent, db.MinionLocation.RelativePoint, db.MinionLocation.X, db.MinionLocation.Y);
			else
				fraMinionAnchor:SetPoint("TOPLEFT", db.MinionParent,"TOPLEFT", 0, -4);
			end
			fraMinionAnchor:SetScale(db.MinionScale);
		end
	end
end

function AdventureTracker:UpdateColourStrings()
	strMinionTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.MinionTitleColour.r * 255, db.Colours.MinionTitleColour.g * 255, db.Colours.MinionTitleColour.b * 255);
	strAdventureItemTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AdventureItemTitleColour.r * 255, db.Colours.AdventureItemTitleColour.g * 255, db.Colours.AdventureItemTitleColour.b * 255);
	strAdventureItemObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AdventureItemObjectiveColour.r * 255, db.Colours.AdventureItemObjectiveColour.g * 255, db.Colours.AdventureItemObjectiveColour.b * 255);
	strAdventureItemObjectivePOIColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AdventureItemObjectivePOIColour.r * 255, db.Colours.AdventureItemObjectivePOIColour.g * 255, db.Colours.AdventureItemObjectivePOIColour.b * 255);
	
end

function AdventureTracker:HandleColourChanges()
	self:UpdateColourStrings()
	if (self:IsVisible() == true) then
		if (blnMinionUpdating == false) then
			blnMinionUpdating = true
			self:ScheduleTimer("UpdateMinion", 0.1)
		end
	end
end

function AdventureTracker:ToggleLockState()
	db.MinionLocked = not db.MinionLocked
end

function AdventureTracker:IsVisible()
	if (self:IsEnabled() == true and dbCore.Main.HideAll == false) then
		return true
	end
	return false	
end
